use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007003
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( examples lib script t xt ) );
__DATA__
Andrews
ApplicationToClass
ApplicationToRole
Attribute
Chris
Class
Composite
Constructor
Cory
Dave
Etheridge
Jesse
Karen
Luehrs
MooseX
Role
Rolsky
UndefTolerant
Watson
at
autarch
chrisandrews
cpan
doy
ether
gphat
irc
lib
